#!/bin/bash

# hrann.sh: Announce the time and temperature every hour.
ident='hrann.sh (V1.4) Wed Dec 23 12:17:16 EST 2015 ground/x'

# Global variables
log_file="$HOME/tmp/hrann.log" # comment this to disable logging
pid_file="$HOME/tmp/hrann.pid"
play_cmd='aplay -q'
digits_path="$HOME/sound_efx/digits-resample"
tmpfile="$HOME/tmp/hrann.tmp.$$"
tts_buffer="$HOME/tmp/vv.txt"
tts_output="$HOME/tmp/vv.wav"
wxtxtfile="$HOME/fc.txt" # Copy of the current forecast kept here.
wxlog="$HOME/fc.log" # Hourly log of weather cond.
active_file="$HOME/tmp/hrann.actlast"
sleep_lvl='4' # sleep for one hour between runs
sleep_pid=''
num_cores='2'

function td {
trap tod_cleanup 2
trap tod_cleanup 15
log 'Starting main loop'
while test -r "$pid_file"
do
sleep_event
sleep 1
hourly_announce
done
log 'Main loop exiting because pid file does not exist.'
exit 0
}
# End function.
function sleep_event {
# Sleep for the amount specified by sleep_lvl variable.
local remaining_min
local remaining_min_sec
local remaining_sec
local sleep_sec
local min_lsd
local raw_min=`date +%M`
local raw_sec=`date +%S`
min_lsd="${raw_min:1}"
if [ "$sleep_lvl" = '0' ]
then
remaining_min='0'
elif [ "$sleep_lvl" = '1' ]
then
if [ "$min_lsd" -ge '5' ]
then
remaining_min=`echo "9-${min_lsd}" |bc`
else
remaining_min=`echo "4-${min_lsd}" |bc`
fi
elif [ "$sleep_lvl" = '2' ]
then
remaining_min=`echo "9-${min_lsd}" |bc`
elif [ "$sleep_lvl" = '3' ]
then
if [ "$raw_min" -ge '30' ]
then
remaining_min=`echo "59-${raw_min}" |bc`
else
remaining_min=`echo "29-${raw_min}" |bc`
fi
elif [ "$sleep_lvl" = '4' ]
then
remaining_min=`echo "59-${raw_min}" |bc`
else
log 'Error: sleep_lvl must be between 0 and 4, exiting.'
exit 1
fi
remaining_min_sec=`echo "${remaining_min}*60" |bc`
remaining_sec=`echo "60-${raw_sec}" |bc`
sleep_sec=`echo "${remaining_sec}+${remaining_min_sec}" |bc`
log "Sleeping for $sleep_sec seconds."
sleep $sleep_sec >& /dev/null &
sleep_pid="$!"
log "Waiting for sleep_pid $sleep_pid"
wait $sleep_pid
log "Returning from sleep event."
return 0
}
# End function.
function log {
if [ "$log_file" != '' ]
then
echo "`date` $$ $*" >> $log_file
fi
return 0
}
# End function.
function tod_cleanup {
# signal handler
log 'Received kill signal, exiting.'
rm -f $pid_file
exit 0
}
# End function.
function say_num {
# Say a number using asterisk  wav files for digits
# Doesn't check to see if this is actually a number so make sure
# before calling.
# Arguments:
# $1 the number to say.
# $2: if we are getting called recursively set to 1
# Avoids saying 50 as fifty zero.
# Function version 20151217
local current_num msd lsd
# Avoid saying leading zeros
current_num=`echo "$1+0" |bc`
msd=`echo "$current_num" |cut -b1`
if [ "$msd" = '-' ]
then
say_digit minus
lsd=`echo "$current_num" |cut -b2-5`
say_num "$lsd" 1
return 0
else
lsd=`echo "$current_num" |cut -b2-4`
fi
if [ "$current_num" = '0' ]
then
if [ "$2" != '1' ]
then
say_digit 0
fi
return 0
fi
if [ "$current_num" -lt 20 ]
then
say_digit "$current_num"
return 0
fi
if [ "$current_num" -lt 100 ]
then
say_digit "${msd}0"
say_num "$lsd" 1
elif [ "$current_num" -lt 1000 ]
then
say_digit "$msd"
say_digit hundred
say_num "$lsd" 1
else
say_digit "$msd"
say_digit thousand
say_num "$lsd" 1
fi
return 0
}
# End function.
function say_digit {
$play_cmd $digits_path/$1.wav
return 0
}
# End function.
function get_field {
# Used to parse the temperature field from wunderground.
local process_line temp_line temp_raw
local field_num='2'
while read process_line
do
if echo "$process_line" |grep -q 'Observed at Lafayette, Indiana'
then
log 'Got line above the one we want'
read temp_line
log "Raw temperature line: $temp_line"
while [ "$field_num" -lt 30 -a "$temp_raw" = '' ]
do
log "trying field $field_num"
temp_raw=`echo "$temp_line" |cut -d " " -f${field_num}`
let field_num="$field_num+1"
done
cat > /dev/null # throw away the rest of the page
echo "$temp_raw"
log "get_field: success reading temperature $temp_raw"
return 0
fi
done
log 'Error could not find temperature line.'
return 1
}
# end function
function read_wx {
# Read and log current weather.
local wxurl wxtemp wxcond wxdate
wxurl='http://braille.wunderground.com/cgi-bin/findweather/getForecast?query=47904'
rm -f "$tts_buffer" "$tts_output" "$tmpfile"
if ! lynx -dump -nolist "$wxurl" |strings > "$tmpfile"
then
log 'lynx exited with an error aborting.'
return 1
fi
rm -f "$wxtxtfile"
cp "$tmpfile" "$wxtxtfile"
chmod 0644 "$wxtxtfile"
rm -f "$tmpfile"
# Grab just the temp from WLFI
# Seems WLFI has a more current temp than does wunderground.
wxtemp=$(lynx -source "http://www.wlfi.com/" \
|grep '<h3 class="weather-capsule-current-temperature">' \
|cut -d '>' -f2 |cut -d '&' -f1 |head -n1)
#wxtemp=`cat "$wxtxtfile" |get_field`
if ! isnum "$wxtemp" 1
then
log 'Error returned temperature was not a number.'
return 1
fi
say_num "$wxtemp"
echo `cat "$wxtxtfile" |grep "Conditions  "`. >> "$tts_buffer"
wxcond=`cat "$tts_buffer" |ltrim`
echo `cat "$wxtxtfile" |grep "Dew Point   "`. >> "$tts_buffer"
#echo `cat "$wxtxtfile" |grep "Pressure   "`. >> "$tts_buffer"
wxdate=`date "+%Y|%m|%d|%H|%M"`
echo "${date}|${wxtemp}|${wxcond}" >> "$wxlog"
check_text "tornado warning"
check_text "tornado watch"
check_text "severe thunderstorm warning"
check_text "severe thunderstorm watch"
check_text "flash flood warning"
check_text "high wind warning"
check_text 'wind advisory'
check_text 'winter storm watch'
check_text 'winter storm warning'
check_text 'blizzard warning'
check_text 'blizzard watch'
check_text 'Special Weather Statement'
check_text 'Winter Weather Advisory'
check_text 'Wind Chill Warning'
speak_buffer
return 0
}
# End function.
function check_text {
# checks the forecast for the specified argument.
# If found we add it to the text to speak.
local text_args="$*"
local text_result=`cat "$wxtxtfile" |grep -i "$text_args" \
|grep -i '.gif]'`
if [ "$text_result" = '' ]
then
return 1
fi
echo "Active $text_args." >> "$tts_buffer"
return 0
}
# End function.
function speak_buffer {
if ! vv "$tts_buffer" "$tmpfile"
then
log 'Error could not run viavoice.'
rm -f "$tmpfile" "$tts_buffer"
return 1
else
sox "$tmpfile" -b 16 "$tts_output" channels 2 rate 48k
$play_cmd "$tts_output"
fi
rm -f "$tmpfile" "$tts_buffer" "$tts_output"
return 0
}
# End function.
function load_avg {
local current_avg
current_avg=`cat /proc/loadavg |cut -d ' ' -f3`
echo "Load average: $current_avg" > "$tts_buffer"
speak_buffer
return 0
}
# End function.
function active_last {
local uptime utf1 utf2 act_current act_last act_final
uptime=`cat /proc/uptime`
utf1=`echo $uptime |cut -d " " -f1`
utf2=`echo $uptime |cut -d " " -f2`
act_current=`echo "$utf1-($utf2/$num_cores)" |bc`
if ! test -r "$active_file"
then
log 'No active file can be read. Writing current value'
echo "$act_current" > "$active_file"
return 0
fi
act_last=`cat "$active_file"`
act_final=`echo "$act_current-$act_last" |bc |cut -d '.' -f1`
log "CPU active $act_final seconds since last run."
echo "c p u: $act_final seconds." > "$tts_buffer"
speak_buffer
echo "$act_current" > "$active_file"
return 0
}
# End function.
function isnum {
# Returns 0 if arg1 is a number 1 if not.
# Special version to handle negative numbers.
# Arguments:
# $1: the number
# $2: if set to 1 allows negative numbers.
# function version 20080310
local numresult num_int num_first_char
# null is not a number
if [ "$1" = "" ]
then
echo 'isnum: number was null.'
return 1
fi
# Special handling if the number zero is passed as 1+0 = 1
if [ "$1" = "0" ]
then
return 0
fi
num_first_char=`echo "$1" |cut -b1`
if [ "$num_first_char" = '-' ]
then
if [ "$2" != '1' ]
then
return 1
fi
num_int=`echo "$1" |cut -b2-99`
else
num_int="$1"
fi
numresult=`echo "1 + $num_int" |bc`
# catch parse errors as they won't happen with numbers
if [ "$numresult" = "" ]
then
log "isnum: Error parsing number: $1"
return 1
fi
if [ "$numresult" = "1" ]
then
return 1
else
return 0
fi
}
# End function.
function ltrim {
# Removes leading spaces from a string
# Gets input on stdin sends to stdout
local input
 local output
local str_started
while read -n 1 input
do
if [ "$str_started" != "1" ]
then
if [ "$input" != "" -o "$input" != " " ]
then
str_started="1"
fi
fi
if [ "$str_started" = "1" ]
then
if [ "$input" = "" ]
then
 # Null is a space in this shit for some reason.
output="${output} "
else
output="${output}${input}"
 fi
continue
else
continue
fi
done
echo $output
return 0
}
# End function
function say_time {
# Speak the current time.
local say_hour say_min min_msd
say_hour=`date +%H`
say_min=`date +%M`
say_num "$say_hour"
if [ "$say_min" = '00' ]
then
say_digit oclock
else
min_msd=`echo "$say_min" |cut -b1`
if [ "$min_msd" = '0' ]
then
say_digit oh
fi
say_num "$say_min"
fi
return 0
}
# End function.
function hourly_announce {
# Function called on the hour.
log 'Doing hourly announcements.'
say_digit hourly
sleep 0.4
say_time
read_wx
#load_avg
active_last
return 0
}
# End function.

# Exec starts here.
# One shot command line options that exit.
if [ "$1" = '-t' ]
then
echo "$ident"
echo 'Running test mode.'
hourly_announce
exit 0
fi
if [ "$1" = '-k' ]
then
if test -r "$pid_file"
then
echo "Killing old process."
kill `cat "$pid_file"`
sleep 1
rm -f $pid_file
else
echo 'Error: no process currently running.'
exit 1
fi
exit 0
fi
log "$ident starting."
if test -r "$pid_file"
then
log "Killing old process."
kill `cat "$pid_file"`
sleep 1
rm -f $pid_file
fi
td >& /dev/null &
td_pid="$!"
echo "Daemon pid: $td_pid"
echo "$td_pid" > "$pid_file"
disown "$td_pid"
exit 0

